package p2.genstack;

/**
 * Run timing benchmarks for various Stack implementations.
 * @author Oscar.Nierstrasz@acm.org
 * @version $Id: StackBenchmark.java 17003 2008-03-13 10:20:41Z oscar $
 */
public class StackBenchmark {
	
	static public void main(String args[]) {
		benchmark(new LinkStack<Integer>());
		// benchmark(new BrokenArrayStack<Integer>());
		benchmark(new ArrayStack<Integer>());
		// benchmark(new SimpleWrappedStack<Integer>());
		benchmark(new WrappedStack<Integer>());
	}
	
	/**
	 * Times how long it takes to do a large number of pushes/pops.
	 */
	static public void benchmark(StackInterface<Integer> stack) {
		int iterations = 100000; // 100K
		Integer item = new Integer(0);
		Timer timer = new Timer();
		long pushTime, popTime;
		
		// System.out.println("Timing " + stack.getClass().getName());
		
		timer.reset();
		for (int i=0; i<iterations; i++) {
			stack.push(item);
		}
		pushTime = timer.timeElapsed();
		// System.out.println(pushTime + " milliseconds for " + iterations + " pushes");
		
		timer.reset();
		for (int i=0; i<iterations; i++) {
			stack.pop();
		}
		popTime = timer.timeElapsed();
		// System.out.println(popTime + " milliseconds for " + iterations + " pops");
		
		String run = "Apple MRJ";
		// String run = "Metrowerks";
		// String run = "Metrowerks JIT";
		System.out.println(run + '\t' + stack.getClass().getName() + '\t'
				+ pushTime + '\t' + popTime);
		
		
	}
	
}
